import pLimit from "p-limit";
export class Runner {
    constructor(options) {
        this._addedTasks = new WeakMap();
        this._limiter = (fn) => fn();
        const concurrency = options?.concurrency;
        if (concurrency !== undefined) {
            this._limiter = pLimit(concurrency);
        }
    }
    async runTasks(...tasks) {
        await Promise.all(tasks.map((task) => {
            const cached = this._addedTasks.get(task);
            if (cached) {
                return cached;
            }
            const promise = this._runTask(task);
            this._addedTasks.set(task, promise);
            return promise;
        }));
    }
    async _runTask(task) {
        this.onTaskAdd?.(task);
        const { dependencies, run } = task.options;
        if (dependencies) {
            await this.runTasks(...dependencies);
        }
        if (!run) {
            return;
        }
        return this._limiter(async () => {
            try {
                this.onTaskStart?.(task);
                await run();
                this.onTaskFinish?.(task);
            }
            catch (e) {
                this.onTaskError?.(task, e);
                throw e;
            }
        });
    }
}
//# sourceMappingURL=runner.js.map